import os
from datetime import datetime

from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver
from webelement_proxy import WebElementProxy


class WebDriverProxy(WebDriver):

    screenshots_path = ""
    __screenshots_enabled = True
    only_pyshot_steps = False

    def __init__(self, webdriver: WebDriver):
        self.__dict__.update(webdriver.__dict__)

    @classmethod
    def get_timestamp(cls) -> str:
        return datetime.now().strftime("%Y-%m-%d_%H-%M-%S_%f")

    @classmethod
    def enable_screenshots(cls):
        cls.__screenshots_enabled = True

    @classmethod
    def disable_screenshots(cls):
        cls.__screenshots_enabled = False

    @classmethod
    def set_screenshots_path(cls, path: str) -> None:
        cls.screenshots_path = path

        if not os.path.exists(path) and path:
            os.makedirs(path)

    def save_screenshot(self, filename="") -> None:
        if self.__screenshots_enabled:
            super().save_screenshot(f"{self.screenshots_path}/{self.get_timestamp()}.png")

    def get(self, url: str) -> None:
        super().get(url)
        self.save_screenshot()

    def find_element(self, by=By.ID, value=None) -> WebElementProxy:
        return WebElementProxy(super().find_element(by, value), self)

    def find_elements(self, by=By.ID, value=None) -> list:
        return [WebElementProxy(element, self) for element in super().find_elements(by, value)]

    def quit(self) -> None:
        self.save_screenshot()
        super().quit()



import configparser
from webdriver_proxy import WebDriverProxy


def pytest_addoption(parser):
    group = parser.getgroup("pyshot", "Facilitate screenshot taking with selenium during testcases")
    group.addoption("--pyshot_conf",
                    action="store",
                    default="pyshot.conf",
                    help="Absolute path to the file pyshot.conf, example. /usr/test/pyshot.conf")


def pyshot_driver(func):
    def wrapper(*args):
        return WebDriverProxy(func(*args))
    return wrapper


def pyshot_step(func):
    def wrapper(*args):

        if WebDriverProxy.only_pyshot_steps:
            WebDriverProxy.enable_screenshots()
            value = func(*args)
            WebDriverProxy.disable_screenshots()
        else:
            value = func(*args)

        return value
    return wrapper


def pytest_runtest_call(item):
    pyshot_conf_path = item.config.getvalue("pyshot_conf")
    default_section = "pyshot"
    parser = configparser.ConfigParser()
    parser.read(pyshot_conf_path)

    screenshots_path = parser.get(default_section, "screenshots_path", fallback="")
    only_pyshot_steps = parser.getboolean(default_section, "only_pyshot_steps", fallback=False)
    create_folder_per_testcase = parser.getboolean(default_section, "create_folder_per_testcase", fallback=True)

    if only_pyshot_steps:
        WebDriverProxy.only_pyshot_steps = True
        WebDriverProxy.disable_screenshots()

    if create_folder_per_testcase and item.name:
        screenshots_path += f"/{item.name[:220]}_{WebDriverProxy.get_timestamp()}"

    if not WebDriverProxy.screenshots_path:
        WebDriverProxy.set_screenshots_path(screenshots_path)

    for value in item.funcargs.values():
        if isinstance(value, WebDriverProxy):
            value.save_screenshot()


def pytest_runtest_teardown(item, nextitem):

    for value in item.funcargs.values():
        if isinstance(value, WebDriverProxy):
            value.save_screenshot()

    WebDriverProxy.set_screenshots_path("")
    WebDriverProxy.only_pyshot_steps = False
    WebDriverProxy.enable_screenshots()



from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement


class WebElementProxy(WebElement):

    def __init__(self, webelement: WebElement, webdriver):
        self.__dict__.update(webelement.__dict__)
        self.webdriver = webdriver

    def find_element(self, by=By.ID, value=None):
        return WebElementProxy(super().find_element(by, value), self.webdriver)

    def find_elements(self, by=By.ID, value=None) -> list:
        return [WebElementProxy(element, self.webdriver) for element in super().find_elements(by, value)]

    def send_keys(self, *value):
        super().send_keys(*value)
        self.webdriver.save_screenshot()

    def click(self):
        super().click()
        self.webdriver.save_screenshot()

    def clear(self):
        super().clear()
        self.webdriver.save_screenshot()



from setuptools import setup

"""
author: anggelomos
github: https://github.com/anggelomos/pyshot
"""
with open("README.md", "r") as readme:
    long_description = readme.read()

setup(
    name="pyshot",
    url="https://github.com/anggelomos/pyshot",
    version="0.0.2",
    author="anggelomos",
    author_email="anggelomos@outlook.com",
    description="Pytest plugin to facilitate screenshot taking with selenium webdriver",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Framework :: Pytest",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python :: 3.8",
    ],
    license="MIT License",
    py_modules=["pyshot", "webdriver_proxy", "webelement_proxy"],
    package_dir={'': 'src'},
    keywords=[
        "pytest", "py.test", "pyshot", "screenshot", "selenium", "test", "plugin"
    ],
    install_requires=[
        "pytest",
        "selenium",
    ],
    entry_points={
        "pytest11": [
            "pyshot = pyshot",
        ]
    }
)
